@@ -73,7 +73,7 @@ static unsigned stringToColour(const ::WPXString &s)
   return val;
 }
 
-libvisio::VSDSVGGenerator::VSDSVGGenerator(libvisio::VSDStringVector &vec): m_gradient(), m_style(), m_gradientIndex(1), m_shadowIndex(1), m_outputSink(), m_vec(vec)
+libvisio::VSDSVGGenerator::VSDSVGGenerator(libvisio::VSDStringVector &vec): m_gradient(), m_style(), m_gradientIndex(1), m_patternIndex(1), m_shadowIndex(1), m_outputSink(), m_vec(vec)
 {
 }
 
@@ -288,11 +288,57 @@ void libvisio::VSDSVGGenerator::setStyle(const ::WPXPropertyList &propList, cons
       }
     }
   }
+  else if(m_style["draw:fill"] && m_style["draw:fill"]->getStr() == "bitmap")
+  {
+    if (m_style["draw:fill-image"] && m_style["libwpg:mime-type"])
+    {
+      m_outputSink << "<svg:defs>\n";
+      m_outputSink << "  <svg:pattern id=\"img" << m_patternIndex++ << "\" patternUnits=\"userSpaceOnUse\" ";
+      if (m_style["svg:width"])
+        m_outputSink << "width=\"" << doubleToString(72*(m_style["svg:width"]->getDouble())) << "\" ";
+      else
+        m_outputSink << "width=\"100\" ";
+
+      if (m_style["svg:height"])
+        m_outputSink << "height=\"" << doubleToString(72*(m_style["svg:height"]->getDouble())) << "\">" << std::endl;
+      else
+        m_outputSink << "height=\"100\">" << std::endl;
+      m_outputSink << "<svg:image ";
+
+      if (m_style["svg:x"])
+        m_outputSink << "x=\"" << doubleToString(72*(m_style["svg:x"]->getDouble())) << "\" ";
+      else
+        m_outputSink << "x=\"0\" ";
+
+      if (m_style["svg:y"])
+        m_outputSink << "y=\"" << doubleToString(72*(m_style["svg:y"]->getDouble())) << "\" ";
+      else
+        m_outputSink << "y=\"0\" ";
+
+      if (m_style["svg:width"])
+        m_outputSink << "width=\"" << doubleToString(72*(m_style["svg:width"]->getDouble())) << "\" ";
+      else
+        m_outputSink << "width=\"100\" ";
+
+      if (m_style["svg:height"])
+        m_outputSink << "height=\"" << doubleToString(72*(m_style["svg:height"]->getDouble())) << "\" ";
+      else
+        m_outputSink << "height=\"100\" ";
+
+      m_outputSink << "xlink:href=\"data:" << m_style["libwpg:mime-type"]->getStr().cstr() << ";base64,";
+      m_outputSink << m_style["draw:fill-image"]->getStr().cstr();
+      m_outputSink << "\" />\n";
+      m_outputSink << "  </svg:pattern>\n";
+      m_outputSink << "</svg:defs>\n";
+    }
+  }
 }
 
 void libvisio::VSDSVGGenerator::startLayer(const ::WPXPropertyList &propList)
 {
-  m_outputSink << "<svg:g id=\"Layer" << propList["svg:id"]->getInt() << "\"";
+  m_outputSink << "<svg:g";
+  if (propList["svg:id"])
+    m_outputSink << " id=\"Layer" << propList["svg:id"]->getInt() << "\"";
   if (propList["svg:fill-rule"])
     m_outputSink << " fill-rule=\"" << propList["svg:fill-rule"]->getStr().cstr() << "\"";
   m_outputSink << " >\n";
@@ -405,7 +451,7 @@ void libvisio::VSDSVGGenerator::drawPath(const ::WPXPropertyListVector &path)
       m_outputSink << (propList["libwpg:sweep"] ? propList["libwpg:sweep"]->getInt() : 1) << " ";
       m_outputSink << doubleToString(72*(propList["svg:x"]->getDouble())) << "," << doubleToString(72*(propList["svg:y"]->getDouble()));
     }
-    else if ((i >= path.count()-1 && i > 2) && propList["libwpg:path-action"] && propList["libwpg:path-action"]->getStr() == "Z" )
+    else if (propList["libwpg:path-action"] && propList["libwpg:path-action"]->getStr() == "Z")
     {
       isClosed = true;
       m_outputSink << "\nZ";
@@ -461,7 +507,6 @@ void libvisio::VSDSVGGenerator::startTextObject(const ::WPXPropertyList &propLis
 {
   double x = 0.0;
   double y = 0.0;
-  double width = 0.0;
   double height = 0.0;
   m_outputSink << "<svg:text ";
   if (propList["svg:x"] && propList["svg:y"])
@@ -475,7 +520,7 @@ void libvisio::VSDSVGGenerator::startTextObject(const ::WPXPropertyList &propLis
 
   if (propList["svg:width"])
   {
-    width = propList["svg:width"]->getDouble();
+    double width = propList["svg:width"]->getDouble();
     xmiddle += width / 2.0;
   }
 
@@ -616,6 +661,9 @@ void libvisio::VSDSVGGenerator::writeStyle(bool /* isClosed */)
   if(m_style["draw:fill"] && m_style["draw:fill"]->getStr() == "gradient")
     m_outputSink << "fill: url(#grad" << m_gradientIndex-1 << "); ";
 
+  if(m_style["draw:fill"] && m_style["draw:fill"]->getStr() == "bitmap")
+    m_outputSink << "fill: url(#img" << m_patternIndex-1 << "); ";
+
   if(m_style["draw:shadow"] && m_style["draw:shadow"]->getStr() == "visible")
     m_outputSink << "filter:url(#shadow" << m_shadowIndex-1 << "); ";
 
